define([
    "dojo/_base/declare",
    "dojo/on",
    "dojo/when",

    // Shell
    "epi",
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/Alert",

    // CMS
    "epi-cms/contentediting/DialogPositionAdjust",
    "epi-cms/_ContentContextMixin",
    "epi-cms/widget/ContentTypeList",
    "epi-cms/contentediting/inline-editing/CreateNewInlineBlockEditFormContainer",
    "epi-cms/contentediting/inline-editing/InlineEditBlockDialog",

    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.editors.contentarea",
    "epi/i18n!epi/cms/nls/episerver.shared.action",
    "epi/i18n!epi/cms/nls/contenttypes.blockdata"
],

function (
    declare,
    on,
    when,

    // Shell
    epi,
    dependency,
    _Command,
    Alert,

    // CMS
    DialogPositionAdjust,
    _ContentContextMixin,
    ContentTypeList,
    CreateNewInlineBlockEditFormContainer,
    InlineEditBlockDialog,

    contentAreaRes,
    sharedRes,
    blockDataRes
) {
    var formIdentifier = "episerver.forms.implementation.elements.formcontainerblock";

    var SimpleContentTypeList = declare([ContentTypeList], {
        _getSuggestedContentTypes: function () {
            return [];
        },
        //TODO: Fix support in EPiServer Forms package, for now we disable the command for forms
        _getAvailableContentTypes: function () {
            var result = this.inherited(arguments);
            return when(result).then(function (types) {
                return types.filter(function (type) {
                    return type.typeIdentifier !== formIdentifier;
                });
            });
        }
    });

    return declare([_Command, _ContentContextMixin], {
        // tags:
        //      internal

        // canExecute: [Boolean]
        //      Default value indicate that command can execute
        canExecute: true,

        // value: [String]
        //      Default select content
        value: null,

        // allowedTypes: [public] Array
        //      The types which are allowed for the given property. i.e used for filtering based on AllowedTypesAttribute
        allowedTypes: null,

        // restrictedTypes: [public] Array
        //      The types which are restricted.
        restrictedTypes: null,

        // suppressOverlayAdjustments: [public] Boolean
        //      Flag which indicates that all overlay adjustments should not be performed. It might be useful when showing another overlay or dialog (e.g. TinyMCE full screen)
        suppressOverlayAdjustments: false,

        postscript: function () {
            this.inherited(arguments);
            this._dialogPositionAdjust = new DialogPositionAdjust();
            this.own(this._dialogPositionAdjust);

            this.contextService = this.contextService || dependency.resolve("epi.shell.ContextService");
        },

        contextChanged: function (ctx, callerData) {
            // summary:
            //      On context change
            // tags:
            //      protected

            this.set("isAvailable", this._isContentContext(ctx));
            this.inherited(arguments);
        },

        _execute: function () {
            // summary:
            //		Executes this command; ...
            // tags:
            //		protected

            when(this.getCurrentContent(), function (currentContent) {
                if (!currentContent) {
                    return;
                }

                // when creating nest block in quick edit
                var contentLink = currentContent.contentLink || currentContent.id;

                // Get the context store for more information about the model
                // We need to append the ?uri= because the request chain can not handle URI's in URI's
                when(this.contextService.query({uri: "epi.cms.contentdata:///" + contentLink}), function (context) {
                    if (!context) {
                        return;
                    }
                    this._createLocalBlock(currentContent, contentLink);
                }.bind(this));
            }.bind(this));
        },

        _createInlineEditDialog: function (content, selectedType) {
            // summary:
            //      Create a dialog for inline-creating a block
            //
            // content:
            //      The content which will be the parent of the new creating block
            // selectedType:
            //      The block type to create the block
            //
            // tags:
            //      private

            var editDialog = new InlineEditBlockDialog({
                title: sharedRes["new"] + " " + contentAreaRes.inline + " " + selectedType.localizedName,
                saveLabel: sharedRes.create,
                suppressOverlayAdjustments: this.suppressOverlayAdjustments
            });

            var contextService = this.contextService;
            editDialog.on("cancel", function () {
                if (!contextService.currentContext) {
                    contextService.currentContext = {};
                }
                contextService.currentContext.currentMode = undefined;
            });

            var form = new CreateNewInlineBlockEditFormContainer({
                autoPublish: true,
                addToDestination: this.model
            }, editDialog.content, "last");

            function updateMainCommandVisibility() {
                if (!editDialog) {
                    return;
                }

                var formValue = form.get("value");
                var allValuesAreFalsy = Object.values(formValue).every(function (x) {
                    return epi.isEmpty(x);
                });

                editDialog.toggleDisabledSaveButton(!form.get("isDirty") || allValuesAreFalsy);
            }

            editDialog.own(on(form, "FormCreated", function () {
                editDialog.show();
                editDialog.toggleMainButton(false);
                editDialog.toggleDisabledSaveButton(false);
                updateMainCommandVisibility();
            }));

            editDialog.own(on(form, "isDirty", updateMainCommandVisibility.bind(form)));

            form.reloadMetadata(content, selectedType.id);

            this.own(on(editDialog, "execute", function () {
                form.saveForm(selectedType);
            }));
            this.own(on(editDialog, "hide", function () {
                this._dialogPositionAdjust.cleanup();
            }.bind(this)));
            editDialog.own(form);
            this.own(editDialog);

            this._dialogPositionAdjust.adjustDialogPosition(editDialog);
        },

        _positionDialogOnTopOfMainContent: function (dialog) {
            // summary:
            //      Content Types List will cover the main editing area with some small padding
            //
            // dialog:
            //      Dialog which will cover the editing area
            //
            // tags:
            //      private

            var mainContent = document.querySelector(".epi-main-content");
            if (mainContent) {
                var dimensions = mainContent.getBoundingClientRect();
                var padding = 40;
                dialog.domNode.style.left = dimensions.left + padding + "px";
                dialog.domNode.style.top = dimensions.top + padding + "px";
                dialog.domNode.style.width = dimensions.width - 2 * padding + "px";
                dialog.domNode.style.height = dimensions.height - 2 * padding + "px";
                dialog.domNode.style.maxWidth = dimensions.width - 2 * padding + "px";
                dialog.containerNode.style.maxHeight = "calc(100% - 100px)";
            }
        },

        _createLocalBlock: function (content, contentLink) {
            var _this = this;
            var dialog;

            var restrictedTypes = this.restrictedTypes;
            var contentTypeList = new SimpleContentTypeList({
                allowedTypes: this.allowedTypes,
                restrictedTypes: restrictedTypes,
                localAsset: true,
                parentLink: contentLink,
                onContentTypeSelected: function (selectedType) {
                    if (dialog) {
                        dialog.hide();
                    }
                    _this._createInlineEditDialog(content, selectedType);
                }
            });

            _this.own(contentTypeList.watch("shouldSkipContentTypeSelection", function () {
                if (contentTypeList.get("shouldSkipContentTypeSelection")) {
                    return;
                }

                dialog = new Alert({
                    // eslint-disable-next-line no-undef
                    acknowledgeActionText: epi.resources.action.close,
                    closeIconVisible: false,
                    draggable: false,
                    content: contentTypeList,
                    dialogClass: "epi-dialog-portrait inline-edit-dialog inline-create-dialog",
                    title: sharedRes["new"] + " " + contentAreaRes.inline + " " + blockDataRes.name
                });

                dialog.show();
                _this._positionDialogOnTopOfMainContent(dialog);
                _this.own(dialog);
                contentTypeList._suggestedContentTypes.setVisibility(false);
            }));
            contentTypeList.set("requestedType", "episerver.core.blockdata");
            contentTypeList.refresh();
            _this.own(contentTypeList);
        }
    });
});
